// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "SceneItem.hpp"
#include "SceneItemComponent.hpp"
#include "scene/components/Components.hpp"
#include "util/array.hpp"

namespace Dawn {
  class Prefab {
    protected:
      virtual std::vector<Asset*> prefabAssets(AssetManager *man) = 0;
      virtual SceneItem * prefabItem(Scene *scene) = 0;

    public:
      /**
       * Returns a list of assets that this prefab requires to be loaded for the
       * instanciation to work.
       * 
       * @param man Asset Manager to retreive the assets from.
       * @return List of required assets, includes sibling/child assets.
       */
      template<class T>
      static std::vector<Asset*> getAssets(AssetManager *man) {
        return T::prefabAssets(man);
      }

      /**
       * Create a scene item from this prefab.
       * 
       * @param scene Scene to add the item to.
       * @return The created scene item for this prefab.
       */
      template<class T>
      static SceneItem * create(Scene *scene) {
        return T::prefabItem(scene);
      }
  };

  class UIPrefab : public Prefab {
    protected:
      /** @deprecated */
      SceneItem * prefabItem(Scene *scene) override {
        assertUnreachable();
      }

      virtual void prefabUIApply(T *existing) = 0;

    public:
      /**
       * Applies a UI Prefab styling to an existing UI Element.
       * 
       * @param existing Existing item to apply styling to.
       */
      static virtual void uiApply(T *existing) {
        V::prefabUIApply(existing);
      }
      
      /**
       * Creates a UI Item from this prefab.
       * 
       * @param canvas Canvas to create this item on to.
       * @return Pointer to the created UI Item.
       */
      static virtual T * uiCreate(UICanvas *canvas) {
        auto item = canvas->addElement<T>();
        V::uiApply(item);
        return item;
      }
  }
}